/*
 * AleatoryVariable.h
 *
 *  Created on: Jul 5, 2009
 *      Author: Alessio Sclocco <alessio@sclocco.eu>
 *
 *     License: See the end of this file for license information
 */

#ifndef ALEATORYVARIABLE_H_
#define ALEATORYVARIABLE_H_

#include <map>
#include <vector>
#include <string>
#include <set>

using std::map;
using std::pair;
using std::vector;
using std::string;
using std::set;

namespace ita {

template< class T > class AleatoryVariable {
public:
	AleatoryVariable();
	~AleatoryVariable();
	void setName(string name);
	string getName() const;
	inline unsigned int getSymbolsNumber() const;
	T getNextSymbol() throw(int);
	void resetNextSymbol();
	double getSymbolProbability(T symbol) const;
	void addSymbol(T symbol, double probability);
	void enqueueInSequence(T symbol);
	T getInSequence(unsigned int i) const;
	inline unsigned int getSequenceLength() const;

private:
	string name;
	bool first;
	map< T, double > values;
	set< T > alph;
	typename set< T >::const_iterator alphIterator;
	vector< T > sequence;
};

template< class T > AleatoryVariable< T >::AleatoryVariable() {
	this->first = true;
}

template< class T >  AleatoryVariable< T >::~AleatoryVariable() {

}

template< class T >  void AleatoryVariable< T >::setName(string name) {
	this->name = name;
}

template< class T >  string AleatoryVariable< T >::getName() const {
	return this->name;
}

template< class T >  inline unsigned int AleatoryVariable< T >::getSymbolsNumber() const {
	return static_cast< unsigned int >((this->alph).size());
}

template< class T > T AleatoryVariable< T >::getNextSymbol() throw(int) {
	T tempSymbol;

	if ( first ) {
		this->alphIterator = this->alph.begin();
		first = false;
	}

	if ( this->alphIterator != this->alph.end() ) {
		tempSymbol = *(this->alphIterator);
		alphIterator++;
		return tempSymbol;
	}
	else {
		throw -1;
	}
}

template< class T >  void AleatoryVariable< T >::resetNextSymbol() {
	this->first = true;
}

template< class T > double AleatoryVariable< T >::getSymbolProbability(T symbol) const {
	typename map< T, double >::const_iterator symbolPosition;

	if ( (symbolPosition = this->values.find(symbol)) != this->values.end() ) {
		return (*symbolPosition).second;
	}
	return 0;
}

template< class T > void AleatoryVariable< T >::addSymbol(T symbol, double probability) {
	this->alph.insert(symbol);
	this->values.insert(pair< T, double >(symbol, probability));
}

template< class T > void AleatoryVariable< T >::enqueueInSequence(T symbol) {
	this->sequence.push_back(symbol);
}

template< class T > T AleatoryVariable< T >::getInSequence(unsigned int i) const {
	return this->sequence[i];
}

template< class T > inline unsigned int AleatoryVariable< T >::getSequenceLength() const {
	return static_cast< unsigned int >((this->sequence).size());
}

}

#endif /* ALEATORYVARIABLE_H_ */

/*
 * Copyright 2009 Alessio Sclocco <alessio@sclocco.eu>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
