/*
 * main.cpp
 *
 *  Created on: Jul 14, 2009
 *      Author: Alessio Sclocco <alessio@sclocco.eu>
 *
 *     License: See the end of this file for license information
 */

#include "include/SampleSetReader.h"
#include "include/SampleSet.h"
#include "include/Entropy.h"
#include <ArgumentList.h>

#include <iostream>
#include <string>
#include <fstream>
#include <set>
#include <limits>

using namespace ita;
using sclocco::ArgumentList;
using std::cout;
using std::cerr;
using std::endl;
using std::string;
using std::ifstream;
using std::ofstream;
using std::set;
using std::numeric_limits;

int main(int argc, char *argv[]) {
	ArgumentList args(argc, argv);

	if ( argc != 7 ) {

		cerr << endl << "Correct usage: " << args.getFirst() << " <[-e] | [-j] | [-c] | [-m]> -n <null_char> <set_file> <input_file> <output_file>" << endl << endl;
		return -1;
	}

	cout << endl << args.getFirst() << ": Information Theory Analizer" << endl;
	bool entropy = args.getSwitch("-e"), joint = args.getSwitch("-j"), conditional = args.getSwitch("-c"), mutual = args.getSwitch("-m");
	string nullSymbol = args.getSwitchArgument("-n"), setFile = args.getFirst(), sampleFile = args.getFirst(), outFile = args.getFirst();

	SampleSet sset;

	/* Loading the sample set */
	SampleSetReader *reader = new SampleSetReader(sampleFile);
	sset.setNullSymbol((nullSymbol.c_str())[0]);
	*reader >> sset;
	delete reader;

	/* Loading sets */
	unsigned int xDim = 0;
	set< unsigned int > x, y;
	ifstream setReader(setFile.c_str());

	setReader >> xDim;

	while ( true ) {
		unsigned int temp = 0;

		setReader >> temp;
		if ( setReader.eof() ) {
			break;
		}

		if ( xDim-- > 0 ) {
			x.insert(temp);
		}
		else {
			y.insert(temp);
		}
	}

	setReader.close();

	/* Perform the chosen operation */
	double result = 0;

	if ( entropy || joint ) {
		result = jointEntropy< char >(x, sset);
	}
	else if ( conditional ) {
		result = conditionalEntropy< char >(x, y, sset);
	}
	else if ( mutual ) {
		result = mutualInformation< char >(x, y, sset);
	}

	/* Write the output to the given file */
	ofstream outWriter(outFile.c_str());
	outWriter << result;
	outWriter.close();

	return 0;
}

/*
 * Copyright 2009 Alessio Sclocco <alessio@sclocco.eu>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
